@@ -872,54 +872,34 @@ out:
 static int read_data_extent(struct metadump_struct *md,
 			    struct async_work *async)
 {
-	struct btrfs_multi_bio *multi = NULL;
-	struct btrfs_device *device;
+	struct btrfs_root *root = md->root;
 	u64 bytes_left = async->size;
 	u64 logical = async->start;
 	u64 offset = 0;
-	u64 bytenr;
 	u64 read_len;
-	ssize_t done;
-	int fd;
+	int num_copies;
+	int cur_mirror;
 	int ret;
 
-	while (bytes_left) {
-		read_len = bytes_left;
-		ret = btrfs_map_block(&md->root->fs_info->mapping_tree, READ,
-				      logical, &read_len, &multi, 0, NULL);
-		if (ret) {
-			fprintf(stderr, "Couldn't map data block %d\n", ret);
-			return ret;
+	num_copies = btrfs_num_copies(&root->fs_info->mapping_tree, logical,
+				      bytes_left);
+
+	/* Try our best to read data, just like read_tree_block() */
+	for (cur_mirror = 0; cur_mirror < num_copies; cur_mirror++) {
+		while (bytes_left) {
+			read_len = bytes_left;
+			ret = read_extent_data(root,
+					(char *)(async->buffer + offset),
+					logical, &read_len, cur_mirror);
+			if (ret < 0)
+				break;
+			offset += read_len;
+			logical += read_len;
+			bytes_left -= read_len;
 		}
-
-		device = multi->stripes[0].dev;
-
-		if (device->fd == 0) {
-			fprintf(stderr,
-				"Device we need to read from is not open\n");
-			free(multi);
-			return -EIO;
-		}
-		fd = device->fd;
-		bytenr = multi->stripes[0].physical;
-		free(multi);
-
-		read_len = min(read_len, bytes_left);
-		done = pread64(fd, async->buffer+offset, read_len, bytenr);
-		if (done < read_len) {
-			if (done < 0)
-				fprintf(stderr, "Error reading extent %d\n",
-					errno);
-			else
-				fprintf(stderr, "Short read\n");
-			return -EIO;
-		}
-
-		bytes_left -= done;
-		offset += done;
-		logical += done;
 	}
-
+	if (bytes_left)
+		return -EIO;
 	return 0;
 }
 
@@ -2608,8 +2588,8 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 	char fs_uuid[BTRFS_UUID_SIZE];
 	u64 devid, type, io_align, io_width;
 	u64 sector_size, total_bytes, bytes_used;
-	char *buf;
-	int fp;
+	char buf[BTRFS_SUPER_INFO_SIZE];
+	int fp = -1;
 	int ret;
 
 	key.objectid = BTRFS_DEV_ITEMS_OBJECTID;
@@ -2619,8 +2599,9 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 	btrfs_init_path(&path);
 	ret = btrfs_search_slot(NULL, info->chunk_root, &key, &path, 0, 0); 
 	if (ret) {
-		fprintf(stderr, "search key fails\n");
-		exit(1);
+		fprintf(stderr, "ERROR: search key failed\n");
+		ret = -EIO;
+		goto out;
 	}
 
 	leaf = path.nodes[0];
@@ -2629,8 +2610,9 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 
 	devid = btrfs_device_id(leaf, dev_item);
 	if (devid != cur_devid) {
-		printk("devid %llu mismatch with %llu\n", devid, cur_devid);
-		exit(1);
+		printk("ERROR: devid %llu mismatch with %llu\n", devid, cur_devid);
+		ret = -EIO;
+		goto out;
 	}
 
 	type = btrfs_device_type(leaf, dev_item);
@@ -2649,15 +2631,9 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 	/* update other devices' super block */
 	fp = open(other_dev, O_CREAT | O_RDWR, 0600);
 	if (fp < 0) {
-		fprintf(stderr, "could not open %s\n", other_dev);
-		exit(1);
-	}
-
-	buf = malloc(BTRFS_SUPER_INFO_SIZE);
-	if (!buf) {
-		ret = -ENOMEM;
-		close(fp);
-		return ret;
+		fprintf(stderr, "ERROR: could not open %s\n", other_dev);
+		ret = -EIO;
+		goto out;
 	}
 
 	memcpy(buf, info->super_copy, BTRFS_SUPER_INFO_SIZE);
@@ -2678,6 +2654,10 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 
 	ret = pwrite64(fp, buf, BTRFS_SUPER_INFO_SIZE, BTRFS_SUPER_INFO_OFFSET);
 	if (ret != BTRFS_SUPER_INFO_SIZE) {
+		if (ret < 0)
+			fprintf(stderr, "ERROR: cannot write superblock: %s\n", strerror(ret));
+		else
+			fprintf(stderr, "ERROR: cannot write superblock\n");
 		ret = -EIO;
 		goto out;
 	}
@@ -2685,9 +2665,9 @@ static int update_disk_super_on_device(struct btrfs_fs_info *info,
 	write_backup_supers(fp, (u8 *)buf);
 
 out:
-	free(buf);
-	close(fp);
-	return 0;
+	if (fp != -1)
+		close(fp);
+	return ret;
 }
 
 static void print_usage(int ret)
