@@ -121,19 +121,10 @@ static const char * const filesystem_cmd_group_usage[] = {
 };
 
 static const char * const cmd_filesystem_df_usage[] = {
-       "btrfs filesystem df [options] <path>",
-       "Show space usage information for a mount point",
-	"-b|--raw           raw numbers in bytes",
-	"-h|--human-readable",
-	"                   human friendly numbers, base 1024 (default)",
-	"-H                 human friendly numbers, base 1000",
-	"--iec              use 1024 as a base (KiB, MiB, GiB, TiB)",
-	"--si               use 1000 as a base (kB, MB, GB, TB)",
-	"-k|--kbytes        show sizes in KiB, or kB with --si",
-	"-m|--mbytes        show sizes in MiB, or MB with --si",
-	"-g|--gbytes        show sizes in GiB, or GB with --si",
-	"-t|--tbytes        show sizes in TiB, or TB with --si",
-       NULL
+	"btrfs filesystem df [options] <path>",
+	"Show space usage information for a mount point",
+	HELPINFO_OUTPUT_UNIT_DF,
+	NULL
 };
 
 static int get_df(int fd, struct btrfs_ioctl_space_args **sargs_ret)
@@ -205,64 +196,14 @@ static int cmd_filesystem_df(int argc, char **argv)
 	int fd;
 	char *path;
 	DIR *dirstream = NULL;
-	unsigned unit_mode = UNITS_DEFAULT;
+	unsigned unit_mode;
 
-	while (1) {
-		int c;
-		static const struct option long_options[] = {
-			{ "raw", no_argument, NULL, 'b'},
-			{ "kbytes", no_argument, NULL, 'k'},
-			{ "mbytes", no_argument, NULL, 'm'},
-			{ "gbytes", no_argument, NULL, 'g'},
-			{ "tbytes", no_argument, NULL, 't'},
-			{ "si", no_argument, NULL, GETOPT_VAL_SI},
-			{ "iec", no_argument, NULL, GETOPT_VAL_IEC},
-			{ "human-readable", no_argument, NULL,
-				GETOPT_VAL_HUMAN_READABLE},
-			{ NULL, 0, NULL, 0 }
-		};
+	unit_mode = get_unit_mode_from_arg(&argc, argv, 1);
 
-		c = getopt_long(argc, argv, "bhHkmgt", long_options, NULL);
-		if (c < 0)
-			break;
-		switch (c) {
-		case 'b':
-			unit_mode = UNITS_RAW;
-			break;
-		case 'k':
-			units_set_base(&unit_mode, UNITS_KBYTES);
-			break;
-		case 'm':
-			units_set_base(&unit_mode, UNITS_MBYTES);
-			break;
-		case 'g':
-			units_set_base(&unit_mode, UNITS_GBYTES);
-			break;
-		case 't':
-			units_set_base(&unit_mode, UNITS_TBYTES);
-			break;
-		case GETOPT_VAL_HUMAN_READABLE:
-		case 'h':
-			unit_mode = UNITS_HUMAN_BINARY;
-			break;
-		case 'H':
-			unit_mode = UNITS_HUMAN_DECIMAL;
-			break;
-		case GETOPT_VAL_SI:
-			units_set_mode(&unit_mode, UNITS_DECIMAL);
-			break;
-		case GETOPT_VAL_IEC:
-			units_set_mode(&unit_mode, UNITS_BINARY);
-			break;
-		default:
-			usage(cmd_filesystem_df_usage);
-		}
-	}
-
-	if (check_argc_exact(argc, optind + 1))
+	if (argc != 2 || argv[1][0] == '-')
 		usage(cmd_filesystem_df_usage);
 
-	path = argv[optind];
+	path = argv[1];
 
 	fd = open_file_or_dir(path, &dirstream);
 	if (fd < 0) {
@@ -815,24 +756,17 @@ fail_out:
 	goto out;
 }
 
-static const char * const cmd_show_usage[] = {
+static const char * const cmd_filesystem_show_usage[] = {
 	"btrfs filesystem show [options] [<path>|<uuid>|<device>|label]",
 	"Show the structure of a filesystem",
 	"-d|--all-devices   show only disks under /dev containing btrfs filesystem",
 	"-m|--mounted       show only mounted btrfs",
-	"--raw              raw numbers in bytes",
-	"--human-readable   human friendly numbers, base 1024 (default)",
-	"--iec              use 1024 as a base (KiB, MiB, GiB, TiB)",
-	"--si               use 1000 as a base (kB, MB, GB, TB)",
-	"--kbytes           show sizes in KiB, or kB with --si",
-	"--mbytes           show sizes in MiB, or MB with --si",
-	"--gbytes           show sizes in GiB, or GB with --si",
-	"--tbytes           show sizes in TiB, or TB with --si",
+	HELPINFO_OUTPUT_UNIT,
 	"If no argument is given, structure of all present filesystems is shown.",
 	NULL
 };
 
-static int cmd_show(int argc, char **argv)
+static int cmd_filesystem_show(int argc, char **argv)
 {
 	LIST_HEAD(all_uuids);
 	struct btrfs_fs_devices *fs_devices;
@@ -845,23 +779,16 @@ static int cmd_show(int argc, char **argv)
 	char path[PATH_MAX];
 	__u8 fsid[BTRFS_FSID_SIZE];
 	char uuid_buf[BTRFS_UUID_UNPARSED_SIZE];
-	unsigned unit_mode = UNITS_DEFAULT;
+	unsigned unit_mode;
 	int found = 0;
 
+	unit_mode = get_unit_mode_from_arg(&argc, argv, 0);
+
 	while (1) {
 		int c;
 		static const struct option long_options[] = {
 			{ "all-devices", no_argument, NULL, 'd'},
 			{ "mounted", no_argument, NULL, 'm'},
-			{ "raw", no_argument, NULL, GETOPT_VAL_RAW},
-			{ "kbytes", no_argument, NULL, GETOPT_VAL_KBYTES},
-			{ "mbytes", no_argument, NULL, GETOPT_VAL_MBYTES},
-			{ "gbytes", no_argument, NULL, GETOPT_VAL_GBYTES},
-			{ "tbytes", no_argument, NULL, GETOPT_VAL_TBYTES},
-			{ "si", no_argument, NULL, GETOPT_VAL_SI},
-			{ "iec", no_argument, NULL, GETOPT_VAL_IEC},
-			{ "human-readable", no_argument, NULL,
-				GETOPT_VAL_HUMAN_READABLE},
 			{ NULL, 0, NULL, 0 }
 		};
 
@@ -875,42 +802,18 @@ static int cmd_show(int argc, char **argv)
 		case 'm':
 			where = BTRFS_SCAN_MOUNTED;
 			break;
-		case GETOPT_VAL_RAW:
-			units_set_mode(&unit_mode, UNITS_RAW);
-			break;
-		case GETOPT_VAL_KBYTES:
-			units_set_base(&unit_mode, UNITS_KBYTES);
-			break;
-		case GETOPT_VAL_MBYTES:
-			units_set_base(&unit_mode, UNITS_MBYTES);
-			break;
-		case GETOPT_VAL_GBYTES:
-			units_set_base(&unit_mode, UNITS_GBYTES);
-			break;
-		case GETOPT_VAL_TBYTES:
-			units_set_base(&unit_mode, UNITS_TBYTES);
-			break;
-		case GETOPT_VAL_SI:
-			units_set_mode(&unit_mode, UNITS_DECIMAL);
-			break;
-		case GETOPT_VAL_IEC:
-			units_set_mode(&unit_mode, UNITS_BINARY);
-			break;
-		case GETOPT_VAL_HUMAN_READABLE:
-			units_set_mode(&unit_mode, UNITS_HUMAN_BINARY);
-			break;
 		default:
-			usage(cmd_show_usage);
+			usage(cmd_filesystem_show_usage);
 		}
 	}
 
 	if (check_argc_max(argc, optind + 1))
-		usage(cmd_show_usage);
+		usage(cmd_filesystem_show_usage);
 
 	if (argc > optind) {
 		search = argv[optind];
 		if (strlen(search) == 0)
-			usage(cmd_show_usage);
+			usage(cmd_filesystem_show_usage);
 		type = check_arg_type(search);
 
 		/*
@@ -1007,20 +910,20 @@ out:
 	return ret;
 }
 
-static const char * const cmd_sync_usage[] = {
+static const char * const cmd_filesystem_sync_usage[] = {
 	"btrfs filesystem sync <path>",
 	"Force a sync on a filesystem",
 	NULL
 };
 
-static int cmd_sync(int argc, char **argv)
+static int cmd_filesystem_sync(int argc, char **argv)
 {
 	int 	fd, res, e;
 	char	*path;
 	DIR	*dirstream = NULL;
 
 	if (check_argc_exact(argc, 2))
-		usage(cmd_sync_usage);
+		usage(cmd_filesystem_sync_usage);
 
 	path = argv[1];
 
@@ -1055,7 +958,7 @@ static int parse_compress_type(char *s)
 	};
 }
 
-static const char * const cmd_defrag_usage[] = {
+static const char * const cmd_filesystem_defrag_usage[] = {
 	"btrfs filesystem defragment [options] <file>|<dir> [<file>|<dir>...]",
 	"Defragment a file or a directory",
 	"",
@@ -1065,7 +968,7 @@ static const char * const cmd_defrag_usage[] = {
 	"-f             flush data to disk immediately after defragmenting",
 	"-s start       defragment only from byte onward",
 	"-l len         defragment only up to len bytes",
-	"-t size        minimal size of file to be considered for defragmenting",
+	"-t size        target extent size hint",
 	NULL
 };
 
@@ -1121,17 +1024,16 @@ error:
 	return 0;
 }
 
-static int cmd_defrag(int argc, char **argv)
+static int cmd_filesystem_defrag(int argc, char **argv)
 {
 	int fd;
 	int flush = 0;
 	u64 start = 0;
 	u64 len = (u64)-1;
-	u32 thresh = 0;
+	u64 thresh = 0;
 	int i;
 	int recursive = 0;
 	int ret = 0;
-	struct btrfs_ioctl_defrag_range_args range;
 	int e = 0;
 	int compress_type = BTRFS_COMPRESS_NONE;
 	DIR *dirstream;
@@ -1170,23 +1072,29 @@ static int cmd_defrag(int argc, char **argv)
 			break;
 		case 't':
 			thresh = parse_size(optarg);
+			if (thresh > (u32)-1) {
+				fprintf(stderr,
+			"WARNING: target extent size %llu too big, trimmed to %u\n",
+					thresh, (u32)-1);
+				thresh = (u32)-1;
+			}
 			defrag_global_fancy_ioctl = 1;
 			break;
 		case 'r':
 			recursive = 1;
 			break;
 		default:
-			usage(cmd_defrag_usage);
+			usage(cmd_filesystem_defrag_usage);
 		}
 	}
 
 	if (check_argc_min(argc - optind, 1))
-		usage(cmd_defrag_usage);
+		usage(cmd_filesystem_defrag_usage);
 
-	memset(&defrag_global_range, 0, sizeof(range));
+	memset(&defrag_global_range, 0, sizeof(defrag_global_range));
 	defrag_global_range.start = start;
 	defrag_global_range.len = len;
-	defrag_global_range.extent_thresh = thresh;
+	defrag_global_range.extent_thresh = (u32)thresh;
 	if (compress_type) {
 		defrag_global_range.flags |= BTRFS_DEFRAG_RANGE_COMPRESS;
 		defrag_global_range.compress_type = compress_type;
@@ -1265,7 +1173,7 @@ static int cmd_defrag(int argc, char **argv)
 	return !!defrag_global_errors;
 }
 
-static const char * const cmd_resize_usage[] = {
+static const char * const cmd_filesystem_resize_usage[] = {
 	"btrfs filesystem resize [devid:][+/-]<newsize>[kKmMgGtTpPeE]|[devid:]max <path>",
 	"Resize a filesystem",
 	"If 'max' is passed, the filesystem will occupy all available space",
@@ -1274,7 +1182,7 @@ static const char * const cmd_resize_usage[] = {
 	NULL
 };
 
-static int cmd_resize(int argc, char **argv)
+static int cmd_filesystem_resize(int argc, char **argv)
 {
 	struct btrfs_ioctl_vol_args	args;
 	int	fd, res, len, e;
@@ -1283,7 +1191,7 @@ static int cmd_resize(int argc, char **argv)
 	struct stat st;
 
 	if (check_argc_exact(argc, 3))
-		usage(cmd_resize_usage);
+		usage(cmd_filesystem_resize_usage);
 
 	amount = argv[1];
 	path = argv[2];
@@ -1322,8 +1230,16 @@ static int cmd_resize(int argc, char **argv)
 	e = errno;
 	close_file_or_dir(fd, dirstream);
 	if( res < 0 ){
-		fprintf(stderr, "ERROR: unable to resize '%s' - %s\n", 
-			path, strerror(e));
+		switch (e) {
+		case EFBIG:
+			fprintf(stderr, "ERROR: unable to resize '%s' - no enouth free space\n",
+				path);
+			break;
+		default:
+			fprintf(stderr, "ERROR: unable to resize '%s' - %s\n",
+				path, strerror(e));
+			break;
+		}
 		return 1;
 	} else if (res > 0) {
 		const char *err_str = btrfs_err_str(res);
@@ -1341,7 +1257,7 @@ static int cmd_resize(int argc, char **argv)
 	return 0;
 }
 
-static const char * const cmd_label_usage[] = {
+static const char * const cmd_filesystem_label_usage[] = {
 	"btrfs filesystem label [<device>|<mount_point>] [<newlabel>]",
 	"Get or change the label of a filesystem",
 	"With one argument, get the label of filesystem on <device>.",
@@ -1349,10 +1265,10 @@ static const char * const cmd_label_usage[] = {
 	NULL
 };
 
-static int cmd_label(int argc, char **argv)
+static int cmd_filesystem_label(int argc, char **argv)
 {
 	if (check_argc_min(argc, 2) || check_argc_max(argc, 3))
-		usage(cmd_label_usage);
+		usage(cmd_filesystem_label_usage);
 
 	if (argc > 2) {
 		return set_label(argv[1], argv[2]);
@@ -1374,12 +1290,18 @@ static const char filesystem_cmd_group_info[] =
 const struct cmd_group filesystem_cmd_group = {
 	filesystem_cmd_group_usage, filesystem_cmd_group_info, {
 		{ "df", cmd_filesystem_df, cmd_filesystem_df_usage, NULL, 0 },
-		{ "show", cmd_show, cmd_show_usage, NULL, 0 },
-		{ "sync", cmd_sync, cmd_sync_usage, NULL, 0 },
-		{ "defragment", cmd_defrag, cmd_defrag_usage, NULL, 0 },
-		{ "balance", cmd_balance, NULL, &balance_cmd_group, 1 },
-		{ "resize", cmd_resize, cmd_resize_usage, NULL, 0 },
-		{ "label", cmd_label, cmd_label_usage, NULL, 0 },
+		{ "show", cmd_filesystem_show, cmd_filesystem_show_usage, NULL,
+			0 },
+		{ "sync", cmd_filesystem_sync, cmd_filesystem_sync_usage, NULL,
+			0 },
+		{ "defragment", cmd_filesystem_defrag,
+			cmd_filesystem_defrag_usage, NULL, 0 },
+		{ "balance", cmd_balance, NULL, &balance_cmd_group,
+			CMD_HIDDEN },
+		{ "resize", cmd_filesystem_resize, cmd_filesystem_resize_usage,
+			NULL, 0 },
+		{ "label", cmd_filesystem_label, cmd_filesystem_label_usage,
+			NULL, 0 },
 		{ "usage", cmd_filesystem_usage,
 			cmd_filesystem_usage_usage, NULL, 0 },
 
